/*  $Id: ATrigger.java 1335 2012-08-25 13:27:49Z sdevaux $
 *
 * Tabs = 4 chars
 * copyright (C) 2002-2012 Sebastien Devaux
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *
 *              Sebastien Devaux <sebastien.devaux@laposte.net>
 */
package net.eduvax.util;

/** 
 * Handle anything with a runnable.
 */
public abstract class ATrigger<T> implements Handler<T>, Runnable
{
	/** Create trigger. */
	protected ATrigger()
	{
		_toRun=null;
	}
	
	/** 
	 * Create trigger.
	 * @param toRun Runnable to execute when candidate matches condition.
	 */
	public ATrigger(Runnable toRun)
	{
		_toRun=toRun;
	}
	/**
	 * Handler implementation.
	 * Object to handle must match a condition (defined with match
	 * method override) to trig the runnable execution.
	 * @param obj object to handle.
	 */ 
	public void handle(T obj)
	{
		if (match(obj))
		{
			run();
		}
	}
	/** 
	 * Runnable implementation.
	 * Execute the runnable. 
	 */
	public void run() 
	{
		if (_toRun!=null)
		{
			_toRun.run();
		}
	}
	/** 
	 * Trig condition.
	 * Implements to define rules controling the execution of the runnable
	 * for each incoming object to handle.
	 */
	protected abstract boolean match(T obj);
	/** 
	 * Defines what the trigger does. 
	 * @djin.relation aggregation
	 */ 
	private Runnable _toRun;
}
